@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2022	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -141,6 +141,12 @@ pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * bac
 	/* Unset suspend reading from frontend flag */
 	pool_unset_suspend_reading_from_frontend();
 
+	/* Initialize where to send map for PREPARE statements */
+#ifdef NOT_USED
+	memset(&session_context->prep_where, 0, sizeof(session_context->prep_where));
+	session_context->prep_where.nelem = POOL_MAX_PREPARED_STATEMENTS;
+#endif							/* NOT_USED */
+
 	/*
 	 * Reset flag to indicate difference in number of affected tuples in
 	 * UPDATE/DELETE.
@@ -162,6 +168,11 @@ pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * bac
 	/* Initialize temp tables */
 	pool_temp_tables_init();
 	
+#ifdef NOT_USED
+	/* Initialize preferred main node id */
+	pool_reset_preferred_main_node_id();
+#endif
+
 	/* Snapshot isolation state */
 	session_context->si_state = SI_NO_SNAPSHOT;
 
@@ -169,12 +180,6 @@ pool_init_session_context(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * bac
 	session_context->transaction_read_only = false;
 
 	dml_adaptive_init();
-
-	unset_tx_started_by_multi_statement_query();
-
-	unset_query_cache_disabled();
-
-	unset_query_cache_disabled_tx();
 }
 
 /*
@@ -943,7 +948,81 @@ pool_copy_prep_where(bool *src, bool *dest)
 {
 	memcpy(dest, src, sizeof(bool) * MAX_NUM_BACKENDS);
 }
+#ifdef NOT_USED
+/*
+ * Add to send map a PREPARED statement
+ */
+void
+pool_add_prep_where(char *name, bool *map)
+{
+	int			i;
 
+	if (!session_context)
+	{
+		ereport(ERROR,
+				(errmsg("pool_add_prep_where: session context is not initialized")));
+		return;
+	}
+
+	for (i = 0; i < POOL_MAX_PREPARED_STATEMENTS; i++)
+	{
+		if (*session_context->prep_where.name[i] == '\0')
+		{
+			strncpy(session_context->prep_where.name[i], name, POOL_MAX_PREPARED_NAME);
+			pool_copy_prep_where(map, session_context->prep_where.where_to_send[i]);
+			return;
+		}
+	}
+	ereport(ERROR,
+			(errmsg("pool_add_prep_where: no empty slot found")));
+}
+
+/*
+ * Search send map by PREPARED statement name
+ */
+bool *
+pool_get_prep_where(char *name)
+{
+	int			i;
+
+	if (!session_context)
+		ereport(ERROR,
+				(errmsg("pool_get_prep_where: session context is not initialized")));
+
+
+	for (i = 0; i < POOL_MAX_PREPARED_STATEMENTS; i++)
+	{
+		if (!strcmp(session_context->prep_where.name[i], name))
+		{
+			return session_context->prep_where.where_to_send[i];
+		}
+	}
+	return NULL;
+}
+
+/*
+ * Remove PREPARED statement by name
+ */
+void
+pool_delete_prep_where(char *name)
+{
+	int			i;
+
+	if (!session_context)
+		ereport(ERROR,
+				(errmsg("pool_delete_prep_where: session context is not initialized")));
+
+	for (i = 0; i < POOL_MAX_PREPARED_STATEMENTS; i++)
+	{
+		if (!strcmp(session_context->prep_where.name[i], name))
+		{
+			memset(&session_context->prep_where.where_to_send[i], 0, sizeof(bool) * MAX_NUM_BACKENDS);
+			*session_context->prep_where.name[i] = '\0';
+			return;
+		}
+	}
+}
+#endif							/* NOT_USED */
 /*
  * Initialize sent message list
  */
@@ -1113,6 +1192,7 @@ pool_pending_message_create(char kind, int len, char *contents)
 	msg->is_rows_returned = false;
 	msg->not_forward_to_frontend = false;
 	memset(msg->node_ids, false, sizeof(msg->node_ids));
+	msg->flush_pending = false;
 
 	MemoryContextSwitchTo(old_context);
 
@@ -1631,6 +1711,24 @@ pool_pending_message_get_message_num_by_backend_id(int backend_id)
 	return cnt;
 }
 
+/*
+ * Set flush request flag
+ */
+void
+pool_pending_message_set_flush_request(void)
+{
+	ListCell   *msg_item;
+
+	foreach(msg_item, session_context->pending_messages)
+	{
+		POOL_PENDING_MESSAGE *msg = (POOL_PENDING_MESSAGE *) lfirst(msg_item);
+		msg->flush_pending = true;
+		ereport(LOG,
+				(errmsg("pool_pending_message_set_flush_request: msg: %s",
+						pool_pending_message_type_to_string(msg->type))));
+	}
+}
+
 /*
  * Dump whole pending message list
  */
@@ -1647,6 +1745,9 @@ dump_pending_message(void)
 		return;
 	}
 
+	if (!message_level_is_interesting(DEBUG5))
+		return;
+
 	ereport(DEBUG5,
 			(errmsg("start dumping pending message list")));
 
@@ -1743,6 +1844,38 @@ pool_unset_suspend_reading_from_frontend(void)
 	session_context->suspend_reading_from_frontend = false;
 }
 
+#ifdef NOT_USED
+/*
+ * Set preferred "main" node id.
+ * Only used for SimpleForwardToFrontend.
+ */
+void
+pool_set_preferred_main_node_id(int node_id)
+{
+	session_context->preferred_main_node_id = node_id;
+}
+
+/*
+ * Return preferred "main" node id.
+ * Only used for SimpleForwardToFrontend.
+ */
+int
+pool_get_preferred_main_node_id(void)
+{
+	return session_context->preferred_main_node_id;
+}
+
+/*
+ * Reset preferred "main" node id.
+ * Only used for SimpleForwardToFrontend.
+ */
+void
+pool_reset_preferred_main_node_id(void)
+{
+	session_context->preferred_main_node_id = -1;
+}
+#endif
+
 /*-----------------------------------------------------------------------
  * Temporary table list management modules.
  *-----------------------------------------------------------------------
@@ -1973,124 +2106,3 @@ pool_temp_tables_dump(void)
 #endif
 
 }
-
-/*
- * Return true if an explicit transaction has been started by a
- * multi-statement-query
- */
-bool
-is_tx_started_by_multi_statement_query(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("is_tx_started_by_multi_statement_query: session context is not initialized")));
-
-	return session_context->is_tx_started_by_multi_statement;
-}
-
-/*
- * Remember that an explicit transaction has been started by a
- * multi-statement-query
- */
-void
-set_tx_started_by_multi_statement_query(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("set_tx_started_by_multi_statement_query: session context is not initialized")));
-
-	session_context->is_tx_started_by_multi_statement = true;
-}
-
-/*
- * Forget that an explicit transaction has been started by a
- * multi-statement-query
- */
-void
-unset_tx_started_by_multi_statement_query(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("unset_tx_started_by_multi_statement_query: session context is not initialized")));
-
-	session_context->is_tx_started_by_multi_statement = false;
-}
-
-/*
- * Set query_cache_disabled
- */
-void
-set_query_cache_disabled(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("set_query_cache_disabled: session context is not initialized")));
-
-	session_context->query_cache_disabled = true;
-}
-
-/*
- * Unset query_cache_disabled
- */
-void
-unset_query_cache_disabled(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("unset_query_cache_disabled: session context is not initialized")));
-
-	session_context->query_cache_disabled = false;
-}
-
-/*
- * Get query_cache_disabled
- */
-bool
-query_cache_disabled(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("query_cache_disabled: session context is not initialized")));
-
-	return session_context->query_cache_disabled ||
-		session_context->query_cache_disabled_tx;
-}
-
-/*
- * Set query_cache_disabled in transaction
- */
-void
-set_query_cache_disabled_tx(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("set_query_cache_disabled_tx: session context is not initialized")));
-
-	session_context->query_cache_disabled_tx = true;
-}
-
-/*
- * Unset query_cache_disabled in transaction
- */
-void
-unset_query_cache_disabled_tx(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("unset_query_cache_disabled_tx: session context is not initialized")));
-
-	session_context->query_cache_disabled_tx = false;
-}
-
-/*
- * Get query_cache_disabled in transaction
- */
-bool
-query_cache_disabled_tx(void)
-{
-	if (!session_context)
-		ereport(ERROR,
-				(errmsg("query_cache_disabled_tx: session context is not initialized")));
-
-	return session_context->query_cache_disabled_tx;
-}
