@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2023	PgPool Global Development Group
+ * Copyright (c) 2003-2022	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -47,6 +47,19 @@
 #define POOLKEYFILE 	".pgpoolkey"
 #define POOLKEYFILEENV "PGPOOLKEYFILE"
 
+/*
+ * Brought from PostgreSQL's pg_config_manual.h.
+ *
+ * Maximum length for identifiers (e.g. table names, column names,
+ * function names).  Names actually are limited to one less byte than this,
+ * because the length must include a trailing zero byte.
+ *
+ * Please note that in version 2 protocol, maximum user name length is
+ * SM_USER, which is 32.
+ */
+#define NAMEDATALEN 64
+
+/* configuration file name */
 #define POOL_CONF_FILE_NAME "pgpool.conf"
 
 /* PCP user/password file name */
@@ -70,6 +83,9 @@
 /* status file name */
 #define STATUS_FILE_NAME "pgpool_status"
 
+/* query cache lock file name */
+#define QUERY_CACHE_LOCK_FILE "memq_lock_file"
+
 /* default string used to identify pgpool on syslog output */
 #define DEFAULT_SYSLOG_IDENT "pgpool"
 
@@ -340,9 +356,9 @@ extern int	my_main_node_id;
 typedef enum
 {
 	POOL_NODE_STATUS_UNUSED,	/* unused */
-	POOL_NODE_STATUS_PRIMARY,	/* primary node */
+	POOL_NODE_STATUS_PRIMARY,	/* primary ndoe */
 	POOL_NODE_STATUS_STANDBY,	/* standby node */
-	POOL_NODE_STATUS_INVALID	/* invalid node (split brain, stand alone) */
+	POOL_NODE_STATUS_INVALID	/* invalid node (split branin, stand alone) */
 }			POOL_NODE_STATUS;
 
 /* Clustering mode macros */
@@ -354,6 +370,7 @@ typedef enum
 #define STREAM (pool_config->backend_clustering_mode == CM_STREAMING_REPLICATION)
 #define LOGICAL (pool_config->backend_clustering_mode == CM_LOGICAL_REPLICATION)
 #define SLONY (pool_config->backend_clustering_mode == CM_SLONY)
+#define DUAL_MODE (REPLICATION || NATIVE_REPLICATION)
 #define RAW_MODE (pool_config->backend_clustering_mode == CM_RAW)
 #define SL_MODE (STREAM || LOGICAL) /* streaming or logical replication mode */
 
@@ -365,16 +382,15 @@ typedef enum
 #define Min(x, y)		((x) < (y) ? (x) : (y))
 
 
-#define MAX_NUM_SEMAPHORES		9
+#define MAX_NUM_SEMAPHORES		8
 #define CONN_COUNTER_SEM		0
 #define REQUEST_INFO_SEM		1
-#define SHM_CACHE_SEM			2
-#define QUERY_CACHE_STATS_SEM	3
-#define PCP_REQUEST_SEM			4
-#define ACCEPT_FD_SEM			5
-#define SI_CRITICAL_REGION_SEM	6
-#define FOLLOW_PRIMARY_SEM		7
-#define MAIN_EXIT_HANDLER_SEM	8	/* used in exit_hander in pgpool main process */
+#define QUERY_CACHE_STATS_SEM	2
+#define PCP_REQUEST_SEM			3
+#define ACCEPT_FD_SEM			4
+#define SI_CRITICAL_REGION_SEM	5
+#define FOLLOW_PRIMARY_SEM		6
+#define MAIN_EXIT_HANDLER_SEM	7	/* used in exit_hander in pgpool main process */
 #define MAX_REQUEST_QUEUE_SIZE	10
 
 #define MAX_SEC_WAIT_FOR_CLUSTER_TRANSACTION 10	/* time in seconds to keep
@@ -494,7 +510,7 @@ typedef enum
 }			POOL_RECOVERY_MODE;
 
 /*
- * Process types.  DO NOT change the order of each enum member!  If you do
+ * Process types.  DO NOT change the order of each enum meber!  If you do
  * that, you must change application_name array in src/main/pgpool_main.c
  * accordingly.
  */
@@ -547,7 +563,6 @@ extern pid_t mypid;				/* parent pid */
 extern pid_t myProcPid;			/* process pid */
 extern ProcessType processType;
 extern ProcessState processState;
-extern bool	reset_query_error;	/* true if error occurs in reset queries */
 extern void set_application_name(ProcessType ptype);
 extern void set_application_name_with_string(char *string);
 extern void set_application_name_with_suffix(ProcessType ptype, int suffix);
@@ -580,6 +595,7 @@ extern BACKEND_STATUS private_backend_status[MAX_NUM_BACKENDS];
 extern char remote_host[];		/* client host */
 extern char remote_port[];		/* client port */
 
+
 /*
  * public functions
  */
@@ -591,7 +607,7 @@ extern char *get_pool_key(void);
 
 
 /*pcp_child.c*/
-extern void pcp_main(int unix_fd, int inet_fd);
+extern void pcp_main(int *fds);
 
 
 
