@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2024	PgPool Global Development Group
+ * Copyright (c) 2003-2023	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -191,7 +191,15 @@ get_config(int *nrows)
 
 	/* - pgpool Connection Settings - */
 	StrNCpy(status[i].name, "listen_addresses", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->listen_addresses);
+	*(status[i].value) = '\0';
+	for (j = 0; j < pool_config->num_listen_addresses; j++)
+	{
+		len = POOLCONFIG_MAXVALLEN - strlen(status[i].value);
+		strncat(status[i].value, pool_config->listen_addresses[j], len);
+		len = POOLCONFIG_MAXVALLEN - strlen(status[i].value);
+		if (j != pool_config->num_listen_addresses - 1)
+			strncat(status[i].value, ",", len);
+	}
 	StrNCpy(status[i].desc, "host name(s) or IP address(es) to listen on", POOLCONFIG_MAXDESCLEN);
 	i++;
 
@@ -200,17 +208,43 @@ get_config(int *nrows)
 	StrNCpy(status[i].desc, "pgpool accepting port number", POOLCONFIG_MAXDESCLEN);
 	i++;
 
-	/* - pgpool Communication Manager Connection Settings - */
-	StrNCpy(status[i].name, "socket_dir", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->socket_dir);
-	StrNCpy(status[i].desc, "pgpool socket directory", POOLCONFIG_MAXDESCLEN);
+	StrNCpy(status[i].name, "unix_socket_directories", POOLCONFIG_MAXNAMELEN);
+	*(status[i].value) = '\0';
+	for (j = 0; j < pool_config->num_unix_socket_directories; j++)
+	{
+		len = POOLCONFIG_MAXVALLEN - strlen(status[i].value);
+		strncat(status[i].value, pool_config->unix_socket_directories[j], len);
+		len = POOLCONFIG_MAXVALLEN - strlen(status[i].value);
+		if (j != pool_config->num_unix_socket_directories - 1)
+			strncat(status[i].value, ",", len);
+	}
+	StrNCpy(status[i].desc, "pgpool socket directories", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	StrNCpy(status[i].name, "unix_socket_group", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->unix_socket_group);
+	StrNCpy(status[i].desc, "owning user of the unix sockets", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	StrNCpy(status[i].name, "unix_socket_permissions", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%04o", pool_config->unix_socket_permissions);
+	StrNCpy(status[i].desc, "access permissions of the unix sockets.", POOLCONFIG_MAXDESCLEN);
 	i++;
 
 	StrNCpy(status[i].name, "pcp_listen_addresses", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->pcp_listen_addresses);
+	*(status[i].value) = '\0';
+	for (j = 0; j < pool_config->num_pcp_listen_addresses; j++)
+	{
+		len = POOLCONFIG_MAXVALLEN - strlen(status[i].value);
+		strncat(status[i].value, pool_config->pcp_listen_addresses[j], len);
+		len = POOLCONFIG_MAXVALLEN - strlen(status[i].value);
+		if (j != pool_config->num_pcp_listen_addresses - 1)
+			strncat(status[i].value, ",", len);
+	}
 	StrNCpy(status[i].desc, "host name(s) or IP address(es) for pcp process to listen on", POOLCONFIG_MAXDESCLEN);
 	i++;
 
+
 	StrNCpy(status[i].name, "pcp_port", POOLCONFIG_MAXNAMELEN);
 	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->pcp_port);
 	StrNCpy(status[i].desc, "PCP port # to bind", POOLCONFIG_MAXDESCLEN);
@@ -301,10 +335,6 @@ get_config(int *nrows)
 	/* POOLS */
 
 	/* - Pool size -  */
-	StrNCpy(status[i].name, "num_init_children", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->num_init_children);
-	StrNCpy(status[i].desc, "# of children initially pre-forked", POOLCONFIG_MAXDESCLEN);
-	i++;
 
 	StrNCpy(status[i].name, "listen_backlog_multiplier", POOLCONFIG_MAXNAMELEN);
 	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->listen_backlog_multiplier);
@@ -326,6 +356,31 @@ get_config(int *nrows)
 	StrNCpy(status[i].desc, "max # of connection pool per child", POOLCONFIG_MAXDESCLEN);
 	i++;
 
+	StrNCpy(status[i].name, "process_management_mode", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->process_management);
+	StrNCpy(status[i].desc, "process management mode", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	StrNCpy(status[i].name, "process_management_strategy", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->process_management_strategy);
+	StrNCpy(status[i].desc, "process management strategy", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	StrNCpy(status[i].name, "num_init_children", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->num_init_children);
+	StrNCpy(status[i].desc, "# of children initially pre-forked", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	StrNCpy(status[i].name, "min_spare_children", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->min_spare_children);
+	StrNCpy(status[i].desc, "min # of spare children waitting for connection", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	StrNCpy(status[i].name, "max_spare_children", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->max_spare_children);
+	StrNCpy(status[i].desc, "max # of spare children waitting for connection", POOLCONFIG_MAXDESCLEN);
+	i++;
+
 	/* - Life time - */
 	StrNCpy(status[i].name, "child_life_time", POOLCONFIG_MAXNAMELEN);
 	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->child_life_time);
@@ -574,6 +629,11 @@ get_config(int *nrows)
 	StrNCpy(status[i].desc, "standby delay threshold", POOLCONFIG_MAXDESCLEN);
 	i++;
 
+	StrNCpy(status[i].name, "delay_threshold_by_time", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->delay_threshold_by_time);
+	StrNCpy(status[i].desc, "standby delay threshold by time", POOLCONFIG_MAXDESCLEN);
+	i++;
+
 	StrNCpy(status[i].name, "prefer_lower_delay_standby", POOLCONFIG_MAXNAMELEN);
 	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%d", pool_config->prefer_lower_delay_standby);
 	StrNCpy(status[i].desc, "load balancing considering streaming delay", POOLCONFIG_MAXDESCLEN);
@@ -781,8 +841,13 @@ get_config(int *nrows)
 	StrNCpy(status[i].desc, "upper server list to observe connection", POOLCONFIG_MAXDESCLEN);
 	i++;
 
-	StrNCpy(status[i].name, "delegate_IP", POOLCONFIG_MAXNAMELEN);
-	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->delegate_IP);
+	StrNCpy(status[i].name, "trusted_server_command", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->trusted_server_command);
+	StrNCpy(status[i].desc, "command executed when upper servers are observed", POOLCONFIG_MAXDESCLEN);
+	i++;
+
+	StrNCpy(status[i].name, "delegate_ip", POOLCONFIG_MAXNAMELEN);
+	snprintf(status[i].value, POOLCONFIG_MAXVALLEN, "%s", pool_config->delegate_ip);
 	StrNCpy(status[i].desc, "delegate IP address of leader pgpool", POOLCONFIG_MAXDESCLEN);
 	i++;
 
@@ -1336,7 +1401,17 @@ get_nodes(int *nrows, int node_id)
 			else
 			{
 				snprintf(nodes[i].role, POOLCONFIG_MAXWEIGHTLEN, "%s", "standby");
-				snprintf(nodes[i].delay, POOLCONFIG_MAXWEIGHTLEN, UINT64_FORMAT, bi->standby_delay);
+
+				if (bi->standby_delay_by_time)
+				{
+					snprintf(nodes[i].delay, POOLCONFIG_MAXWEIGHTLEN, "%.6f", ((float)bi->standby_delay)/1000000);
+					if (bi->standby_delay >= 2*1000*1000)
+						strcat(nodes[i].delay, " seconds");
+					else
+						strcat(nodes[i].delay, " second");
+				}
+				else
+					snprintf(nodes[i].delay, POOLCONFIG_MAXWEIGHTLEN, UINT64_FORMAT, bi->standby_delay);
 			}
 
 			if (!strcmp(nodes[i].pg_status, "up"))
@@ -1454,11 +1529,11 @@ get_pools(int *nrows)
 		for (pool = 0; pool < pool_config->max_pool; pool++)
 		{
 			int idle_duration = pi->connection_info[pool * MAX_NUM_BACKENDS].client_idle_duration;
-			int client_idle_time = pool_config->client_idle_limit;
+			int cliet_idle_time = pool_config->client_idle_limit;
 
 			if (pool_config->client_idle_limit > 0)
 			{
-				client_idle_time = pool_config->client_idle_limit - idle_duration;
+				cliet_idle_time = pool_config->client_idle_limit - idle_duration;
 			}
 
 			for (backend_id = 0; backend_id < NUM_BACKENDS; backend_id++)
@@ -1523,8 +1598,8 @@ get_pools(int *nrows)
 				{
 					snprintf(pools[lines].client_idle_duration, sizeof(pools[lines].client_idle_duration),
 							 "%d (%d:%02d before client disconnected)", idle_duration,
-							 client_idle_time / 60,
-							 client_idle_time % 60);
+							 cliet_idle_time / 60,
+							 cliet_idle_time % 60);
 				}
 				else
 					snprintf(pools[lines].client_idle_duration, sizeof(pools[lines].client_idle_duration),
@@ -1570,6 +1645,9 @@ get_pools(int *nrows)
 					case IDLE_IN_TRANS:
 						StrNCpy(pools[lines].status, "Idle in transaction", POOLCONFIG_MAXPROCESSSTATUSLEN);
 						break;
+					case CONNECTING:
+						StrNCpy(pools[lines].status, "Connecting", POOLCONFIG_MAXPROCESSSTATUSLEN);
+						break;
 					default:
 						*(pools[lines].status) = '\0';
 				}
@@ -1694,6 +1772,9 @@ get_processes(int *nrows)
 			case IDLE_IN_TRANS:
 				StrNCpy(processes[child].status, "Idle in transaction", POOLCONFIG_MAXPROCESSSTATUSLEN);
 				break;
+			case CONNECTING:
+				StrNCpy(processes[child].status, "Connecting", POOLCONFIG_MAXPROCESSSTATUSLEN);
+				break;
 			default:
 				*(processes[child].status) = '\0';
 		}
@@ -1731,6 +1812,8 @@ processes_reporting(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend)
 	num_fields = sizeof(field_names) / sizeof(char *);
 	processes = get_processes(&nrows);
 
+	send_row_description(frontend, backend, num_fields, field_names);
+
 	send_row_description_and_data_rows(frontend, backend, num_fields, field_names, offsettbl,
 									   (char *)processes, sizeof(POOL_REPORT_PROCESSES), nrows);
 
@@ -1765,6 +1848,8 @@ version_reporting(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend)
 	num_fields = sizeof(field_names) / sizeof(char *);
 	version = get_version();
 
+	send_row_description(frontend, backend, num_fields, field_names);
+
 	send_row_description_and_data_rows(frontend, backend, num_fields, field_names, offsettbl,
 									   (char *)version, sizeof(POOL_REPORT_VERSION), nrows);
 
@@ -1805,7 +1890,7 @@ cache_reporting(POOL_CONNECTION * frontend, POOL_CONNECTION_POOL * backend)
 	 * Get raw cache stat data
 	 */
 	POOL_SETMASK2(&BlockSig, &oldmask);
-	pool_shmem_lock();
+	pool_shmem_lock(POOL_MEMQ_SHARED_LOCK);
 
 	PG_TRY();
 	{
@@ -2348,7 +2433,7 @@ char *db_node_role(int node)
 	/*
 	 * Query whether the node is in recovery.
 	 */
-	if (get_query_result(slots, node, "SELECT pg_catalog.pg_is_in_recovery()", &res))
+	if (get_query_result(slots, node, "SELECT pg_is_in_recovery()", &res))
 	{
 		return "unknown";
 	}
